mod ctrl_c;
mod models;

use std::{
    borrow::Cow,
    ffi::{CStr, CString},
    thread::sleep,
    time::Duration,
};

use ctrl_c::CtrlC;
use models::{AtcID, AtcIDCPacked, Fuel};
// use simconnect_rs::SimConnect;
use simconnect_sdk::{Notification, SimConnect, SystemEventRequest};

fn main() -> Result<(), anyhow::Error> {
    use tracing::Level;
    use tracing_subscriber::layer::SubscriberExt;
    use tracing_subscriber::util::SubscriberInitExt;
    use tracing_subscriber::Layer;

    let fmt = tracing_subscriber::fmt::layer().with_filter(tracing_subscriber::filter::filter_fn(
        |metadata| metadata.level() < &Level::TRACE,
    ));

    tracing_subscriber::registry().with(fmt).init();

    let Ok(mut simconnect) = SimConnect::new("Avii's Virtual Airline Manager") else {
        panic!("Game not running, gotta wait for it here");
    };

    // .subscribe(SomeEvent);

    let ctrl_c = CtrlC::new();

    loop {
        if let Ok(Some(event)) = simconnect.get_next_dispatch() {
            match event {
                Notification::Open => {
                    simconnect.subscribe_to_system_event(SystemEventRequest::FlightLoaded)?;

                    let id = simconnect.register_object::<AtcID>()?;
                    simconnect.register_object::<Fuel>()?;

                    let cs = CString::new("ITWORKS")?;
                    let mut buffer = [0i8; 256];
                    for (i, b) in cs.to_bytes_with_nul().iter().enumerate() {
                        buffer[i] = *b as i8;
                    }

                    simconnect
                        .set_data_on_sim_object_with_id(id, &mut AtcIDCPacked { value: buffer })?;
                    tracing::info!("ATC_ID set");

                    simconnect.set_data_on_sim_object(&mut Fuel {
                        ft1: 0.0,
                        wt1: 10.0,
                        wt2: 0.0,
                        et1: 0.0,
                        et2: 0.0,
                        et3: 0.0,
                        ft2: 0.0,
                    })?;
                    tracing::info!("FUEL set");
                }
                Notification::Object(object) => {
                    // if let Ok(fuel) = Fuel::try_from(&object) {
                    //     // dbg!(fuel);
                    //     continue;
                    // }
                }
                Notification::SystemEvent(system_event) => {
                    if let simconnect_sdk::SystemEvent::FlightLoaded { .. } = system_event {
                        let cs = CString::new("ITWORKS")?;
                        let mut buffer = [0i8; 256];
                        for (i, b) in cs.to_bytes_with_nul().iter().enumerate() {
                            buffer[i] = *b as i8;
                        }

                        let id = simconnect.get_object_id::<AtcID>()?;
                        simconnect.set_data_on_sim_object_with_id(
                            id,
                            &mut AtcIDCPacked { value: buffer },
                        )?;
                        tracing::info!("ATC_ID set");

                        simconnect.set_data_on_sim_object(&mut Fuel {
                            ft1: 0.0,
                            wt1: 10.0,
                            wt2: 0.0,
                            et1: 0.0,
                            et2: 0.0,
                            et3: 0.0,
                            ft2: 0.0,
                        })?;
                        tracing::info!("FUEL set");
                    }
                }
                Notification::Quit => panic!("Sim Quit"),
                _ => todo!(),
            }
        }

        sleep(Duration::from_millis(1000));

        if ctrl_c.count() > 0 {
            break;
        }
    }

    drop(simconnect);

    Ok(())
}
